<?php


if (!defined('ABSPATH')) {
    exit;
}

// ✅ Fetch API Key & Assistant ID from the database
define('OPENAI_API_KEY', get_option('wp_chatbot_openai_api_key', '')); // Default empty if not set
define('ASSISTANT_ID', get_option('wp_chatbot_assistant_id', '')); // Default empty if not set

// Check if API credentials exist
if (empty(OPENAI_API_KEY) || empty(ASSISTANT_ID)) {
    error_log("Chatbot Error: OpenAI API Key or Assistant ID is missing.");
    wp_send_json_error(['error' => 'OpenAI API Key or Assistant ID is missing. Please configure it in the settings.']);
}

// Create a new thread when the chatbox loads
add_action('wp_ajax_wp_chatbot_create_thread', 'wp_chatbot_create_thread');
add_action('wp_ajax_nopriv_wp_chatbot_create_thread', 'wp_chatbot_create_thread');

function wp_chatbot_create_thread() {
    if (isset($_COOKIE['chatbot_thread_id'])) {
        // ✅ Reuse existing thread ID if it's still valid
        $thread_id = sanitize_text_field($_COOKIE['chatbot_thread_id']);
        error_log("✅ Reusing existing thread: " . $thread_id);
        wp_send_json_success(['thread_id' => $thread_id]);
    }

    // ✅ No valid thread found, create a new one
    $api_url = "https://api.openai.com/v1/threads";
    $headers = [
        "Authorization" => "Bearer " . OPENAI_API_KEY,
        "OpenAI-Beta" => "assistants=v2",
        "Content-Type" => "application/json"
    ];

    $response = wp_remote_post($api_url, [
        'headers' => $headers,
        'method'  => 'POST',
        'body'    => json_encode([])
    ]);

    if (is_wp_error($response)) {
        error_log("❌ Failed to create thread: " . $response->get_error_message());
        wp_send_json_error(['error' => 'Failed to create thread.']);
    }

    $response_body = json_decode(wp_remote_retrieve_body($response), true);

    if (!isset($response_body['id'])) {
        error_log("❌ Invalid OpenAI response: " . print_r($response_body, true));
        wp_send_json_error(['error' => 'Invalid response from OpenAI.', 'full_response' => $response_body]);
    }

    $thread_id = $response_body['id'];

    // ✅ Set the thread ID as a cookie (expires in 30 minutes)
    setcookie("chatbot_thread_id", $thread_id, time() + (30 * 60), "/"); 

    error_log("✅ New thread created: " . $thread_id);
    wp_send_json_success(['thread_id' => $thread_id]);
}


// Handle sending user messages
add_action('wp_ajax_wp_chatbot_send', 'wp_chatbot_send_message');
add_action('wp_ajax_nopriv_wp_chatbot_send', 'wp_chatbot_send_message');

function wp_chatbot_send_message() {
    if (!isset($_POST['message']) || !isset($_POST['thread_id'])) {
        wp_send_json_error(['message' => 'Missing message or thread ID']);
        wp_die();
    }

    $apiKey = get_option('wp_chatbot_openai_api_key');
    $assistantId = get_option('wp_chatbot_assistant_id');
    $userMessage = sanitize_text_field($_POST['message']);
    $threadId = sanitize_text_field($_POST['thread_id']);

    // ✅ Step 1: Store the User's Message in the Thread
    $messagePayload = array(
        'role' => 'user',
        'content' => $userMessage
    );

    $messageResponse = wp_remote_post("https://api.openai.com/v1/threads/" . $threadId . "/messages", array(
        'method'    => 'POST',
        'headers'   => array(
            'Authorization' => 'Bearer ' . $apiKey,
            'Content-Type'  => 'application/json',
            'OpenAI-Beta'   => 'assistants=v2'
        ),
        'body'      => json_encode($messagePayload)
    ));

    if (is_wp_error($messageResponse)) {
        wp_send_json_error(['message' => 'Failed to send message to OpenAI']);
        wp_die();
    }

    // ✅ Step 2: Run the Assistant on the Thread
    $runPayload = array(
        'assistant_id' => $assistantId
    );

    $runResponse = wp_remote_post("https://api.openai.com/v1/threads/" . $threadId . "/runs", array(
        'method'    => 'POST',
        'headers'   => array(
            'Authorization' => 'Bearer ' . $apiKey,
            'Content-Type'  => 'application/json',
            'OpenAI-Beta'   => 'assistants=v2'
        ),
        'body'      => json_encode($runPayload)
    ));

    $runData = json_decode(wp_remote_retrieve_body($runResponse), true);
    $runId = isset($runData['id']) ? $runData['id'] : null;

    if (!$runId) {
        wp_send_json_error(['message' => 'Failed to start assistant run']);
        wp_die();
    }

    // ✅ Step 3: Poll for Assistant Completion (Up to 10 attempts)
    $responseReceived = false;
    $attempts = 0;

    while (!$responseReceived && $attempts < 20) {
        sleep(2); // Wait 2 seconds before checking

        $checkResponse = wp_remote_get("https://api.openai.com/v1/threads/" . $threadId . "/runs/" . $runId, array(
            'headers'   => array(
                'Authorization' => 'Bearer ' . $apiKey,
                'Content-Type'  => 'application/json',
                'OpenAI-Beta'   => 'assistants=v2'
            )
        ));

        $checkData = json_decode(wp_remote_retrieve_body($checkResponse), true);

        if (isset($checkData['status']) && $checkData['status'] === 'completed') {
            $responseReceived = true;
        } elseif (isset($checkData['status']) && in_array($checkData['status'], array('failed', 'cancelled'))) {
            wp_send_json_error(['message' => 'Assistant run failed']);
            wp_die();
        }

        $attempts++;
    }

    if (!$responseReceived) {
        wp_send_json_error(['message' => 'Assistant run did not complete in time']);
        wp_die();
    }

    // ✅ Step 4: Retrieve Assistant's Response
    $messagesResponse = wp_remote_get("https://api.openai.com/v1/threads/" . $threadId . "/messages", array(
        'headers'   => array(
            'Authorization' => 'Bearer ' . $apiKey,
            'Content-Type'  => 'application/json',
            'OpenAI-Beta'   => 'assistants=v2'
        )
    ));

    $messagesData = json_decode(wp_remote_retrieve_body($messagesResponse), true);
    $messages = isset($messagesData['data']) ? $messagesData['data'] : array();
    $assistantMessage = '';

    // ✅ Step 5: Extract the Latest Assistant Message
    foreach ($messages as $msg) {
        if ($msg['role'] === 'assistant') {
            $assistantMessage = implode("\n", array_map(function ($c) {
                return isset($c['text']['value']) ? $c['text']['value'] : '';
            }, $msg['content']));
            break; // Stop at the first assistant response found (latest one)
        }
    }

    // ✅ Step 6: Send Response Back to Chatbot
    wp_send_json_success([
        'response' => $assistantMessage,
        'timeout'  => 30,
    ]);

    wp_die();
}

add_action('wp_ajax_wp_chatbot_fetch_history', 'wp_chatbot_fetch_history');
add_action('wp_ajax_nopriv_wp_chatbot_fetch_history', 'wp_chatbot_fetch_history');

function wp_chatbot_fetch_history() {
    if (!isset($_POST['thread_id'])) {
        error_log("❌ Chatbot Error: Missing thread ID.");
        wp_send_json_error(['error' => 'Missing thread ID.']);
    }

    $thread_id = sanitize_text_field($_POST['thread_id']);
    $api_url = "https://api.openai.com/v1/threads/{$thread_id}/messages";

    $headers = [
        "Authorization" => "Bearer " . OPENAI_API_KEY,
        "OpenAI-Beta" => "assistants=v2",
        "Content-Type" => "application/json"
    ];

    $response = wp_remote_get($api_url, ['headers' => $headers]);

    if (is_wp_error($response)) {
        error_log("❌ OpenAI API Request Failed: " . $response->get_error_message());
        wp_send_json_error(['error' => 'Failed to fetch chat history.']);
    }

    $response_body = json_decode(wp_remote_retrieve_body($response), true);
error_log("❌ body messages array: " . print_r($response_body,true));
    if (!isset($response_body['data']) || !is_array($response_body['data'])) {
        error_log("❌ Invalid OpenAI Response: " . print_r($response_body, true));
        wp_send_json_error(['error' => 'Invalid response from OpenAI.', 'full_response' => $response_body]);
    }

    $messages = [];
    foreach ($response_body['data'] as $msg) {
        if (isset($msg['content']) && isset($msg['role'])) {
            $messages[] = $msg['content'][0]['text']['value'] ?? '';
        }
    }

    wp_send_json_success(['messages' => $messages]);
}

?>